/*
** pork_format.c - helper functions for formatting text.
** Copyright (C) 2002 Amber Adams <amber@ojnk.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#include <config.h>

#include <unistd.h>
#include <ncurses.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>

#include <libfaim/aim.h>
#include <libfaim/faimconfig.h>

#include <pork_util.h>
#include <pork_list.h>
#include <pork_buddy.h>
#include <pork_set.h>
#include <pork_swindow.h>
#include <pork_imwindow.h>
#include <pork_slist.h>
#include <pork_buddy_list.h>
#include <pork_acct.h>
#include <pork_cstr.h>
#include <pork_misc.h>
#include <pork_input.h>
#include <pork_bind.h>
#include <pork_screen.h>
#include <pork_screen_cmd.h>
#include <pork_format.h>
#include <pork_color.h>

extern struct screen screen;

void make_timestamp(const char *work_format, char *buf, size_t len) {
	u_int32_t i;
	u_int32_t s = 0;
	u_int32_t time_12hour = 0;
	time_t cur_time;
	struct tm *tm;
	char tmp[CHAR_SHORTBUF];

	cur_time = time(NULL);
	tm = localtime(&cur_time);

	if (tm == NULL) {
		*buf = '\0';
		return;
	}

	len--;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			*tmp = '\0';
			switch (work_format[++i]) {
				case 'H':
					snprintf(tmp, sizeof(tmp), "%02d", tm->tm_hour);
					break;

				case 'h':
					if (tm->tm_hour == 0)
						time_12hour = 12;
					else {
						time_12hour = tm->tm_hour;

						if (time_12hour > 12)
							time_12hour -= 12;
					}

					snprintf(tmp, sizeof(tmp), "%02d", time_12hour);
					break;

				case 'm':
					snprintf(tmp, sizeof(tmp), "%02d", tm->tm_min);
					break;

				case 's':
					snprintf(tmp, sizeof(tmp), "%02d", tm->tm_sec);
					break;

				case 'z':
					snprintf(tmp, sizeof(tmp), "%s",
						(tm->tm_hour >= 12) ? "pm" : "am");
					break;
			}

			if (*tmp != '\0') {
				u_int32_t j = 0;

				while (tmp[j] != '\0' && j < sizeof(tmp) && s < len)
					buf[s++] = tmp[j++];
			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}

	buf[s] = '\0';
}

void make_activity(const char *work_format, char *buf, size_t len) {
	u_int32_t i;
	u_int32_t s = 0;
	dlist_t *cur = screen.window_list;
	u_int32_t activity = 0;
	char digit[CHAR_SHORTBUF];
	char tmp[CHAR_SHORTBUF];

	*tmp = '\0';
	len--;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			if (work_format[++i] == 'A' && !activity) {
				do {
					struct imwindow *imwindow = cur->data;

					if (imwindow->swindow.activity &&
						!imwindow->swindow.visible)
					{
						if (activity > 0) {
							snprintf(digit, sizeof(tmp),
								",%u", imwindow->refnum);
						} else {
							snprintf(digit, sizeof(tmp),
								"%u", imwindow->refnum);
						}

						xstrncat(tmp, digit, sizeof(tmp));
						activity++;
					}

					cur = cur->next;

				} while (cur != screen.window_list);

				if (activity) {
					u_int32_t j = 0;

					while (tmp[j] != '\0' && j < sizeof(tmp) && s < len)
						buf[s++] = tmp[j++];
				} else {
					*buf = '\0';
					return;
				}
			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}

	buf[s] = '\0';
}

void make_typing(	const char *work_format,
					char *buf,
					size_t len,
					struct imwindow *imwindow)
{
	u_int32_t i;
	u_int32_t s = 0;

	len--;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			if (work_format[++i] == 'Y') {
				char *tmp;
				u_int32_t j = 0;

				tmp = NULL;

				if (imwindow->typing) {
					if (imwindow->typing == 1)
						tmp = opt_get_str(OPT_TEXT_TYPING_PAUSED);
					else
						tmp = opt_get_str(OPT_TEXT_TYPING);

					while (tmp[j] != '\0' && j < sizeof(tmp) && s < len)
						buf[s++] = tmp[j++];
				} else {
					*buf = '\0';
					return;
				}
			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}

	buf[s] = '\0';
}

void make_held(	const char *work_format,
				char *buf,
				size_t len,
				struct imwindow *imwindow)
{
	char tmp[CHAR_SHORTBUF];
	u_int32_t i;
	u_int32_t j;
	u_int32_t s = 0;

	len--;

	*buf = '\0';

	if (imwindow->swindow.held == 0)
		return;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			if (work_format[++i] == 'H') {
				snprintf(tmp, CHAR_SHORTBUF, "%02d", imwindow->swindow.held);

				for (j = 0 ; tmp[j] != '\0' && s < len ; j++)
					buf[s++] = tmp[j];

			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}
	buf[s] = '\0';
}

void make_im(	const char *work_format,
				char *buf,
				size_t len,
				const char *nick,
				const char *msg)
{
	u_int32_t i;
	u_int32_t s = 0;
	const char *tmp;

	len--;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			switch (work_format[++i]) {
				case 'n':
				case 'N':
					tmp = nick;
					break;

				case 'm':
				case 'M':
					tmp = msg;
					break;

				default:
					tmp = NULL;
					break;
			}

			if (tmp != NULL) {
				u_int32_t j = 0;

				while (tmp[j] != '\0' && s < len)
					buf[s++] = tmp[j++];
			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}

	buf[s] = '\0';
}
